/* Copyright (C) 2005 Free Software Foundation, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Kang Jeong-Hee <Keizi@mail.co.kr>
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>

#include <glib/gi18n-lib.h>
#include <glib/gmessages.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libnautilus-extension/nautilus-extension-types.h>
#include <libnautilus-extension/nautilus-menu-provider.h>

#define FOLDER_OF_FORTUNE_TYPE (folder_of_fortune_get_type ())
#define FOLDER_OF_FORTUNE(o)   (G_TYPE_CHECK_INSTANCE_CAST ((o), FOLDER_OF_FORTUNE_TYPE))

typedef struct {
	GObject parent;
} FolderOfFortune;

typedef struct {
	GObjectClass parent_class;
} FolderOfFortuneClass;

static GType		this_type = 0;
static GObjectClass	*parent_class = NULL;

static GType
folder_of_fortune_get_type (void)
{
    return this_type;
}

/* FIXME */
static gchar *
get_fortunate_name (void)
{
    gchar name[1024];

    g_sprintf (name, "FIXME_%d", g_random_int_range (0, 100));

    return name;
}

static void
create_folder (NautilusFileInfo *current_folder)
{
    gchar *parent_dir, *new, *parent_path, *new_path, *full;

    parent_dir = NAUTILUS_FILE_INFO_GET_IFACE (current_folder)->get_uri (current_folder);
    new = get_fortunate_name ();

    parent_path = g_filename_from_uri (parent_dir, NULL, NULL);
    new_path = g_filename_from_utf8 (new, -1, NULL, NULL, NULL);
    full = g_strconcat (parent_path, G_DIR_SEPARATOR_S, new_path, NULL);

    if (g_mkdir (full, 0777) < 0) {
        g_debug ("Error creating folder : %s", full);
    }

    g_free (full);
    g_free (new_path);
    g_free (parent_path);
}

static void
context_menu_activate (NautilusMenuItem *item,
		       NautilusFileInfo *current_folder)
{
    create_folder (current_folder);
}

static GList *
folder_of_fortune_get_background_items (NautilusMenuProvider *provider,
					GtkWidget *window,
					NautilusFileInfo *current_folder)
{
    GList *items = NULL;
    NautilusMenuItem *item;

    /* create the context menu item */
    item = nautilus_menu_item_new ("Folder of fortune::create_any_folder",
				   _("Create any folder"),
				   _("Create a new folder of random name"),
				   NULL);
    g_signal_connect (item, "activate",
    		      G_CALLBACK (context_menu_activate), current_folder);
    items = g_list_prepend (items, item);

    return items;
}

static void
folder_of_fortune_class_init (FolderOfFortuneClass *class)
{
    parent_class = g_type_class_peek_parent (class);
}
static void
folder_of_fortune_init (FolderOfFortune *self)
{
}
static void
menu_provider_iface_init (NautilusMenuProviderIface *iface)
{
    iface->get_background_items = folder_of_fortune_get_background_items;
}

void
folder_of_fortune_register_type (GTypeModule *module)
{
    static const GTypeInfo info = {
	sizeof (FolderOfFortuneClass),
	(GBaseInitFunc) NULL,
	(GBaseFinalizeFunc) NULL,
	(GClassInitFunc) folder_of_fortune_class_init,
	NULL,
	NULL,
	sizeof (FolderOfFortune),
	0,
	(GInstanceInitFunc) folder_of_fortune_init
    };
    static const GInterfaceInfo menu_provider_iface_info = {
	(GInterfaceInitFunc)menu_provider_iface_init,
	NULL,
	NULL
    };

    this_type = g_type_module_register_type (module,
					     G_TYPE_OBJECT,
					     "FolderOfFortune",
					     &info, 0);
    g_type_module_add_interface (module,
				 this_type,
				 NAUTILUS_TYPE_MENU_PROVIDER,
				 &menu_provider_iface_info);
}

void
nautilus_module_initialize (GTypeModule *module)
{
    folder_of_fortune_register_type (module);

    bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
    textdomain (GETTEXT_PACKAGE);
}

void
nautilus_module_shutdown (void)
{
}

void
nautilus_module_list_types (const GType **types,
			    int          *num_types)
{
    static GType type_list[1];

    type_list[0] = FOLDER_OF_FORTUNE_TYPE;
    *types = type_list;
    *num_types = G_N_ELEMENTS (type_list);
}
